@@ -22,6 +22,8 @@
 #include "../globals.h"
 #include "XMLwrapper.h"
 
+namespace zyn {
+
 #define rStdString(name, len, ...) \
     {STRINGIFY(name) "::s", rMap(length, len) rProp(parameter) DOC(__VA_ARGS__), NULL, rStringCb(name,len)}
 #define rStdStringCb(name, length) rBOIL_BEGIN \
@@ -120,8 +122,8 @@ static const rtosc::Ports ports = {
     //rArrayS(cfg.presetsDirList,MAX_BANK_ROOT_DIRS),
     rToggle(cfg.CheckPADsynth, "Old Check For PADsynth functionality within a patch"),
     rToggle(cfg.IgnoreProgramChange, "Ignore MIDI Program Change Events"),
-    rParamI(cfg.UserInterfaceMode,   "Beginner/Advanced Mode Select"),
-    rParamI(cfg.VirKeybLayout,       "Keyboard Layout For Virtual Piano Keyboard"),
+    rParamI(cfg.UserInterfaceMode, "Beginner/Advanced Mode Select"),
+    rParamI(cfg.VirKeybLayout, "Keyboard Layout For Virtual Piano Keyboard"),
     //rParamS(cfg.LinuxALSAaudioDev),
     //rParamS(cfg.nameTag)
     {"cfg.OscilPower::i", rProp(parameter) rDoc("Size Of Oscillator Wavetable"), 0,
@@ -148,7 +150,7 @@ static const rtosc::Ports ports = {
             }
 
         }},
-    {"favorites:", rProp(parameter), 0,
+    {"favorites:", /*rProp(parameter)*/ 0, 0,
         [](const char *msg, rtosc::RtData &d)
         {
             Config &c = *(Config*)d.obj;
@@ -168,11 +170,13 @@ static const rtosc::Ports ports = {
             delete [] args;
         }},
 };
-const rtosc::Ports &Config::ports = ::ports;
+const rtosc::Ports &Config::ports = zyn::ports;
 #endif
 
 Config::Config()
-{}
+{
+    init();
+}
 
 void Config::init()
 {
@@ -449,3 +453,5 @@ void Config::getConfigFileName(char *name, int namesize) const
     name[0] = 0;
     snprintf(name, namesize, "%s%s", getenv("HOME"), "/.zynaddsubfxXML.cfg");
 }
+
+}
