@@ -16,6 +16,7 @@
 
 #include "Part.h"
 
+#include "zyn-version.h"
 #include "../Misc/Stereo.h"
 #include "../Misc/Util.h"
 #include "../Params/LFOParams.h"
@@ -32,15 +33,18 @@
 #include <stdio.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <fstream>
 #include <iostream>
 #include <algorithm>
 #include <cmath>
 #include <atomic>
-
 #include <unistd.h>
 
 using namespace std;
 using namespace rtosc;
+
+namespace zyn {
+
 #define rObject Master
 
 static const Ports sysefxPort =
@@ -49,12 +53,18 @@ static const Ports sysefxPort =
         rDoc("gain on part to sysefx routing"), 0,
         [](const char *m, RtData&d)
         {
-            //ok, this is going to be an ugly workaround
-            //we know that if we are here the message previously MUST have
-            //matched Psysefxvol#/
-            //and the number is one or two digits at most
-            const char *index_1 = m;
-            index_1 -=2;
+            //we know that if we are here the location must
+            //be ...Psysefxvol#N/part#M
+            //and the number "N" is one or two digits at most
+
+            // go backto the '/'
+            const char* m_findslash = m + strlen(m),
+                      * loc_findslash = d.loc + strlen(d.loc);
+            for(;*loc_findslash != '/'; --m_findslash, --loc_findslash)
+                assert(*loc_findslash == *m_findslash);
+            assert(m_findslash + 1 == m);
+
+            const char *index_1 = loc_findslash-1;
             assert(isdigit(*index_1));
             if(isdigit(index_1[-1]))
                 index_1--;
@@ -78,9 +88,15 @@ static const Ports sysefsendto =
     {"to#" STRINGIFY(NUM_SYS_EFX) "::i",
         rProp(parameter) rDoc("sysefx to sysefx routing gain"), 0, [](const char *m, RtData&d)
         {
-            //same ugly workaround as before
-            const char *index_1 = m;
-            index_1 -=2;
+            //same workaround as before
+            //go backto the '/'
+            const char* m_findslash = m + strlen(m),
+                      * loc_findslash = d.loc + strlen(d.loc);
+            for(;*loc_findslash != '/'; --m_findslash, --loc_findslash)
+                assert(*loc_findslash == *m_findslash);
+            assert(m_findslash + 1 == m);
+
+            const char *index_1 = loc_findslash-1;
             assert(isdigit(*index_1));
             if(isdigit(index_1[-1]))
                 index_1--;
@@ -98,6 +114,236 @@ static const Ports sysefsendto =
         }}
 };
 
+#define rBegin [](const char *msg, RtData &d) { rtosc::AutomationMgr &a = *(AutomationMgr*)d.obj
+#define rEnd }
+
+static int extract_num(const char *&msg)
+{
+    while(*msg && !isdigit(*msg)) msg++;
+    int num = atoi(msg);
+    while(isdigit(*msg)) msg++;
+    return num;
+}
+
+static int get_next_int(const char *msg)
+{
+    return extract_num(msg);
+}
+
+static const Ports mapping_ports = {
+    {"offset::f", rProp(parameter) rShort("off") rLinear(-50, 50) rMap(unit, percent), 0,
+        rBegin;
+        int slot = d.idx[1];
+        int param = d.idx[0];
+        if(!strcmp("f",rtosc_argument_string(msg))) {
+            a.setSlotSubOffset(slot, param, rtosc_argument(msg, 0).f);
+            a.updateMapping(slot, param);
+            d.broadcast(d.loc, "f", a.getSlotSubOffset(slot, param));
+        } else
+            d.reply(d.loc, "f", a.getSlotSubOffset(slot, param));
+        rEnd},
+    {"gain::f", rProp(parameter) rShort("gain") rLinear(-200, 200) rMap(unit, percent), 0,
+        rBegin;
+        int slot = d.idx[1];
+        int param = d.idx[0];
+        if(!strcmp("f",rtosc_argument_string(msg))) {
+            a.setSlotSubGain(slot, param, rtosc_argument(msg, 0).f);
+            a.updateMapping(slot, param);
+            d.broadcast(d.loc, "f", a.getSlotSubGain(slot, param));
+        } else
+            d.reply(d.loc, "f", a.getSlotSubGain(slot, param));
+        rEnd},
+};
+
+static const Ports auto_param_ports = {
+    {"used::T:F", rProp(parameter) rProp(read-only) rDoc("If automation is assigned to anything"), 0,
+        rBegin;
+        int slot  = d.idx[1];
+        int param = d.idx[0];
+
+        d.reply(d.loc, a.slots[slot].automations[param].used ? "T" : "F");
+        rEnd},
+    {"active::T:F", rProp(parameter) rDoc("If automation is being actively used"), 0,
+        rBegin;
+        int slot  = d.idx[1];
+        int param = d.idx[0];
+        if(rtosc_narguments(msg))
+            a.slots[slot].automations[param].active = rtosc_argument(msg, 0).T;
+        else
+            d.reply(d.loc, a.slots[slot].automations[param].active ? "T" : "F");
+        rEnd},
+    {"path::s", rProp(parameter) rProp(read-only) rDoc("Path of parameter"), 0,
+        rBegin;
+        int slot  = d.idx[1];
+        int param = d.idx[0];
+        d.reply(d.loc, "s", a.slots[slot].automations[param].param_path);
+        rEnd},
+    {"clear:", 0, 0,
+        rBegin;
+        int slot = d.idx[1];
+        int param = d.idx[0];
+        a.clearSlotSub(slot, param);
+        rEnd},
+    {"mapping/", 0, &mapping_ports,
+        rBegin;
+        SNIP;
+        mapping_ports.dispatch(msg, d);
+        rEnd},
+
+    //{"mapping", rDoc("Parameter mapping control"), 0,
+    //    rBegin;
+    //    int slot  = d.idx[1];
+    //    int param = d.idx[0];
+    //    if(!strcmp("b", rtosc_argument_string(msg))) {
+    //        int    len  = rtosc_argument(msg, 0).b.len / sizeof(float);
+    //        float *data = (float*)rtosc_argument(msg, 0).b.data;
+    //    } else {
+    //        d.reply(d.loc, "b",
+    //                a.slots[slot].automations[param].map.npoints*sizeof(float),
+    //                a.slots[slot].automations[param].map.control_points);
+    //    }
+    //    rEnd},
+};
+
+static const Ports slot_ports = {
+    //{"learn-binding:s", rDoc("Create binding for automation path with midi-learn"), 0,
+    //    rBegin;
+    //    (void) m;
+    //    //m->automate.createBinding(rtosc_argument(msg, 0).i,
+    //    //                          rtosc_argument(msg, 1).s,
+    //    //                          rtosc_argument(msg, 2).T);
+    //    rEnd},
+    //{"create-binding:s", rDoc("Create binding for automation path"), 0,
+    //    rBegin;
+    //    m->automate.createBinding(rtosc_argument(msg, 0).i,
+    //                              rtosc_argument(msg, 1).s,
+    //                              rtosc_argument(msg, 2).T);
+    //    rEnd},
+    {"value::f", rProp(parameter) rMap(default, 0.5) rLinear(0, 1) rDoc("Access current value in slot 'i' (0..1)"), 0,
+        rBegin;
+        int num = d.idx[0];
+        if(!strcmp("f",rtosc_argument_string(msg))) {
+            a.setSlot(num, rtosc_argument(msg, 0).f);
+            d.broadcast(d.loc, "f", a.getSlot(num));
+        } else
+            d.reply(d.loc, "f", a.getSlot(num));
+        rEnd},
+
+    {"name::s", rProp(parameter) rDoc("Access name of automation slot"), 0,
+        rBegin;
+        int num = d.idx[0];
+        if(!strcmp("s",rtosc_argument_string(msg))) {
+            a.setName(num, rtosc_argument(msg, 0).s);
+            d.broadcast(d.loc, "s", a.getName(num));
+        } else
+            d.reply(d.loc, "s", a.getName(num));
+        rEnd},
+    {"midi-cc::i", rProp(parameter) rMap(default, -1) rDoc("Access assigned midi CC slot") , 0,
+        rBegin;
+        int slot = d.idx[0];
+        if(rtosc_narguments(msg))
+            a.slots[slot].midi_cc = rtosc_argument(msg, 0).i;
+        else
+            d.reply(d.loc, "i", a.slots[slot].midi_cc);
+
+        rEnd},
+    {"active::T:F",  rProp(parameter) rMap(default, F) rDoc("If Slot is enabled"), 0,
+        rBegin;
+        int slot = d.idx[0];
+        if(rtosc_narguments(msg))
+            a.slots[slot].active = rtosc_argument(msg, 0).T;
+        else
+            d.reply(d.loc, a.slots[slot].active ? "T" : "F");
+        rEnd},
+    {"learning::i", rProp(parameter) rMap(default, -1) rDoc("If slot is trying to find a midi learn binding"), 0,
+        rBegin;
+        int slot = d.idx[0];
+        d.reply(d.loc, "i", a.slots[slot].learning);
+        rEnd},
+    {"clear:", 0, 0,
+        rBegin;
+        int slot = d.idx[0];
+        a.clearSlot(slot);
+        rEnd},
+    {"param#4/", rDoc("Info on individual param mappings"), &auto_param_ports,
+        rBegin;
+        (void)a;
+        d.push_index(get_next_int(msg));
+        SNIP;
+        auto_param_ports.dispatch(msg, d);
+        d.pop_index();
+        rEnd},
+};
+
+static const Ports automate_ports = {
+    {"active-slot::i", rProp(parameter) rMap(min, -1) rMap(max, 16) rDoc("Active Slot for macro learning"), 0,
+        rBegin;
+        if(!strcmp("i",rtosc_argument_string(msg))) {
+            a.active_slot = rtosc_argument(msg, 0).i;
+            d.broadcast(d.loc, "i", a.active_slot);
+        } else
+            d.reply(d.loc, "i", a.active_slot);
+        rEnd},
+    {"learn-binding-new-slot:s", rDoc("Learn a parameter assigned to a new slot"), 0,
+        rBegin;
+        int free_slot = a.free_slot();
+        if(free_slot >= 0) {
+            a.createBinding(free_slot, rtosc_argument(msg, 0).s, true);
+            a.active_slot = free_slot;
+        }
+        rEnd},
+    {"learn-binding-same-slot:s", rDoc("Learn a parameter appending to the active-slot"), 0,
+        rBegin;
+        if(a.active_slot >= 0)
+            a.createBinding(a.active_slot, rtosc_argument(msg, 0).s, true);
+        rEnd},
+    // TODO: remove rNoWalk
+    {"slot#16/", rNoWalk rDoc("Parameters of individual automation slots"), &slot_ports,
+        rBegin;
+        (void)a;
+        d.push_index(get_next_int(msg));
+        SNIP;
+        slot_ports.dispatch(msg, d);
+        d.pop_index();
+        rEnd},
+    {"clear", 0, 0,
+        rBegin;
+        for(int i=0; i<a.nslots; ++i)
+            a.clearSlot(i);
+        rEnd},
+    {"load-blob:b", rProp(internal) rDoc("Load blob from middleware"), 0,
+        rBegin;
+        auto &b = **(rtosc::AutomationMgr **)rtosc_argument(msg, 0).b.data;
+        //XXX this code should likely be in rtosc
+        for(int i=0; i<a.nslots; ++i) {
+            auto &slota = a.slots[i];
+            auto &slotb = b.slots[i];
+            std::swap(slota.learning, slotb.learning);
+            std::swap(slota.midi_cc,  slotb.midi_cc);
+            std::swap(slota.used,     slotb.used);
+            std::swap(slota.active,   slotb.active);
+            for(int j=0; j<a.per_slot; ++j) {
+                auto &aa = slota.automations[j];
+                auto &ab = slotb.automations[j];
+                std::swap(aa.used, ab.used);
+                std::swap(aa.active, ab.active);
+                std::swap(aa.param_path, ab.param_path);
+                std::swap(aa.param_min, ab.param_min);
+                std::swap(aa.param_max, ab.param_max);
+                std::swap(aa.param_step, ab.param_step);
+                std::swap(aa.param_type, ab.param_type);
+                std::swap(aa.map.offset, ab.map.offset);
+                std::swap(aa.map.gain, ab.map.gain);
+                std::swap(aa.map.upoints, ab.map.upoints);
+                for(int k=0; k<aa.map.npoints; ++k)
+                    std::swap(aa.map.control_points[k], ab.map.control_points[k]);
+            }
+        }
+        rEnd},
+};
+
+#undef  rBegin
+#undef  rEnd
 #define rBegin [](const char *msg, RtData &d) { Master *m = (Master*)d.obj
 #define rEnd }
 
@@ -108,20 +354,22 @@ static const Ports watchPorts = {
         rEnd},
 };
 
+
 extern const Ports bankPorts;
 static const Ports master_ports = {
     rString(last_xmz, XMZ_PATH_MAX, "File name for last name loaded if any."),
     rRecursp(part, 16, "Part"),//NUM_MIDI_PARTS
     rRecursp(sysefx, 4, "System Effect"),//NUM_SYS_EFX
     rRecursp(insefx, 8, "Insertion Effect"),//NUM_INS_EFX
-    rRecur(microtonal, "Micrtonal Mapping Functionality"),
+    rRecur(microtonal, "Microtonal Mapping Functionality"),
     rRecur(ctl, "Controller"),
-    rArrayI(Pinsparts, NUM_INS_EFX, rOpt(-2, Master), rOpt(-1, Off)
-            rOptions(Part1, Part2, Part3, Part4,  Part5, Part6,
-                 Part7, Part8, Part9, Part10, Part11, Part12,
-                 Part13, Part14, Part15, Part16),
-                "Part to insert part onto"),
-    {"Pkeyshift::i", rShort("key shift") rProp(parameter) rLinear(0,127) rDoc("Global Key Shift"), 0, [](const char *m, RtData&d) {
+    rArrayOption(Pinsparts, NUM_INS_EFX, rOpt(-2, Master), rOpt(-1, Off),
+                 rOptions(Part1, Part2, Part3, Part4, Part5, Part6,
+                          Part7, Part8, Part9, Part10, Part11, Part12,
+                          Part13, Part14, Part15, Part16) rDefault(Off),
+                 "Part to insert part onto"),
+    {"Pkeyshift::i", rShort("key shift") rProp(parameter) rLinear(0,127)
+        rDefault(64) rDoc("Global Key Shift"), 0, [](const char *m, RtData&d) {
         if(rtosc_narguments(m)==0) {
             d.reply(d.loc, "i", ((Master*)d.obj)->Pkeyshift);
         } else if(rtosc_narguments(m)==1 && rtosc_type(m,0)=='i') {
@@ -160,6 +408,8 @@ static const Ports master_ports = {
        d.reply("/free", "sb", "Part", sizeof(void*), &m->part[i]);
        m->part[i] = p;
        p->initialize_rt();
+       for(int i=0; i<128; ++i)
+           m->activeNotes[i] = 0;
        }},
     {"active_keys:", rProp("Obtain a list of active notes"), 0,
         rBegin;
@@ -168,14 +418,16 @@ static const Ports master_ports = {
             keys[i] = m->activeNotes[i] ? 'T' : 'F';
         d.broadcast(d.loc, keys);
         rEnd},
-    {"Pvolume::i", rShort("volume") rProp(parameter) rLinear(0,127) rDoc("Master Volume"), 0,
+    {"Pvolume::i", rShort("volume") rProp(parameter) rLinear(0,127)
+        rDefault(80) rDoc("Master Volume"), 0,
         [](const char *m, rtosc::RtData &d) {
         if(rtosc_narguments(m)==0) {
             d.reply(d.loc, "i", ((Master*)d.obj)->Pvolume);
         } else if(rtosc_narguments(m)==1 && rtosc_type(m,0)=='i') {
             ((Master*)d.obj)->setPvolume(limit<char>(rtosc_argument(m,0).i,0,127));
             d.broadcast(d.loc, "i", ((Master*)d.obj)->Pvolume);}}},
-    {"volume::i", rShort("volume") rProp(parameter) rLinear(0,127) rDoc("Master Volume"), 0,
+    {"volume::i", rShort("volume") rProp(parameter) rLinear(0,127)
+        rDefault(80) rDoc("Master Volume"), 0,
         [](const char *m, rtosc::RtData &d) {
         if(rtosc_narguments(m)==0) {
             d.reply(d.loc, "i", ((Master*)d.obj)->Pvolume);
@@ -229,13 +481,12 @@ static const Ports master_ports = {
         [](const char *,RtData &d) {
             Master *M =  (Master*)d.obj;
             M->frozenState = false;}},
-    {"midi-learn/", 0, &rtosc::MidiMapperRT::ports,
+    {"automate/", rDoc("MIDI Learn/Plugin Automation support"), &automate_ports,
         [](const char *msg, RtData &d) {
-            Master *M =  (Master*)d.obj;
             SNIP;
-            printf("residue message = <%s>\n", msg);
-            d.obj = &M->midi;
-            rtosc::MidiMapperRT::ports.dispatch(msg,d);}},
+            d.obj = (void*)&((Master*)d.obj)->automate;
+            automate_ports.dispatch(msg, d);
+            }},
     {"close-ui:", rDoc("Request to close any connection named \"GUI\""), 0,
         [](const char *, RtData &d) {
        d.reply("/close-ui", "");}},
@@ -261,8 +512,8 @@ static const Ports master_ports = {
         [](const char *, rtosc::RtData &d) {d.reply("/undo_pause", "");}},
     {"undo_resume:",rProp(internal) rDoc("resume undo event recording"),0,
         [](const char *, rtosc::RtData &d) {d.reply("/undo_resume", "");}},
-    {"config/", rDoc("Top Level Application Configuration Parameters"), &Config::ports,
-        [](const char *, rtosc::RtData &d){d.forward();}},
+    {"config/", rNoWalk rDoc("Top Level Application Configuration Parameters"),
+        &Config::ports, [](const char *, rtosc::RtData &d){d.forward();}},
     {"presets/", rDoc("Parameter Presets"), &preset_ports, rBOIL_BEGIN
         SNIP
             preset_ports.dispatch(msg, data);
@@ -286,6 +537,14 @@ static const Ports master_ports = {
         rBOIL_END},
     {"bank/", rDoc("Controls for instrument banks"), &bankPorts,
             [](const char*,RtData&) {}},
+    {"learn:s", rProp(depricated) rDoc("MIDI Learn"), 0,
+        rBegin;
+        int free_slot = m->automate.free_slot();
+        if(free_slot >= 0) {
+            m->automate.createBinding(free_slot, rtosc_argument(msg, 0).s, true);
+            m->automate.active_slot = free_slot;
+        }
+        rEnd},
 };
 
 #undef rBegin
@@ -359,18 +618,97 @@ vuData::vuData(void)
       rmspeakl(0.0f), rmspeakr(0.0f), clipped(0)
 {}
 
+void Master::saveAutomation(XMLwrapper &xml, const rtosc::AutomationMgr &midi)
+{
+    xml.beginbranch("automation");
+    {
+        XmlNode metadata("mgr-info");
+        metadata["nslots"]       = to_s(midi.nslots);
+        metadata["nautomations"] = to_s(midi.per_slot);
+        metadata["ncontrol"]     = to_s(midi.slots[0].automations[0].map.npoints);
+        xml.add(metadata);
+
+        for(int i=0; i<midi.nslots; ++i) {
+            const auto &slot = midi.slots[i];
+            if(!slot.used)
+                continue;
+            xml.beginbranch("slot", i);
+            XmlNode params("params");
+            params["midi-cc"] = to_s(slot.midi_cc);
+            xml.add(params);
+            for(int j=0; j<midi.per_slot; ++j) {
+                const auto &au = slot.automations[j];
+                if(!au.used)
+                    continue;
+                xml.beginbranch("automation", j);
+                XmlNode automation("params");
+                automation["path"] = au.param_path;
+                XmlNode mapping("mapping");
+                mapping["gain"]   = to_s(au.map.gain);
+                mapping["offset"] = to_s(au.map.offset);
+                xml.add(automation);
+                xml.add(mapping);
+                xml.endbranch();
+            }
+
+            xml.endbranch();
+        }
+    }
+    xml.endbranch();
+}
+
+void Master::loadAutomation(XMLwrapper &xml, rtosc::AutomationMgr &midi)
+{
+    if(xml.enterbranch("automation")) {
+        for(int i=0; i<midi.nslots; ++i) {
+            auto &slot = midi.slots[i];
+            if(xml.enterbranch("slot", i)) {
+                for(int j=0; j<midi.per_slot; ++j) {
+                    auto &au = slot.automations[j];
+                    if(xml.enterbranch("automation", j)) {
+                        float gain       = 1.0;
+                        float offset     = 0.0;
+                        std::string path = "";
+                        for(auto node:xml.getBranch()) {
+                            if(node.name == "params")
+                                path = node["path"];
+                            else if(node.name == "mapping") {
+                                gain   = atof(node["gain"].c_str());
+                                offset = atof(node["offset"].c_str());
+                            }
+                        }
+                        printf("createBinding(%d, %s, false)\n", i, path.c_str());
+                        midi.createBinding(i, path.c_str(), false);
+                        midi.setSlotSubGain(i, j, gain);
+                        midi.setSlotSubOffset(i, j, offset);
+                        xml.exitbranch();
+                    }
+                }
+                for(auto node:xml.getBranch())
+                    if(node.name == "params")
+                        slot.midi_cc = atoi(node["midi-cc"].c_str());
+                xml.exitbranch();
+            }
+        }
+        xml.exitbranch();
+    }
+}
+
 Master::Master(const SYNTH_T &synth_, Config* config)
     :HDDRecorder(synth_), time(synth_), ctl(synth_, &time),
     microtonal(config->cfg.GzipCompression), bank(config),
+    automate(16,4,8),
     frozenState(false), pendingMemory(false),
     synth(synth_), gzip_compression(config->cfg.GzipCompression)
 {
     bToU = NULL;
     uToB = NULL;
 
-    //Setup MIDI
-    midi.frontend = [this](const char *msg) {bToU->raw_write(msg);};
-    midi.backend  = [this](const char *msg) {applyOscEvent(msg);};
+    //Setup MIDI Learn
+    automate.set_ports(master_ports);
+    automate.set_instance(this);
+    //midi.frontend = [this](const char *msg) {bToU->raw_write(msg);};
+    automate.backend  = [this](const char *msg) {applyOscEvent(msg);};
 
     memory = new AllocatorClass();
     swaplr = 0;
@@ -441,6 +779,7 @@ void Master::defaults()
 
     for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart) {
         part[npart]->defaults();
+        part[npart]->partno  = npart % NUM_MIDI_CHANNELS;
         part[npart]->Prcvchn = npart % NUM_MIDI_CHANNELS;
     }
 
@@ -519,8 +858,7 @@ void Master::setController(char chan, int type, int par)
 {
     if(frozenState)
         return;
-    //TODO add chan back
-    midi.handleCC(type,par);
+    automate.handleMidi(chan, type, par);
     if((type == C_dataentryhi) || (type == C_dataentrylo)
        || (type == C_nrpnhi) || (type == C_nrpnlo)) { //Process RPN and NRPN by the Master (ignore the chan)
         ctl.setparameternumber(type, par);
@@ -724,13 +1062,19 @@ bool Master::runOSC(float *outl, float *outr, bool offline)
         }
         if(!d.matches) {// && !ports.apropos(msg)) {
             fprintf(stderr, "%c[%d;%d;%dm", 0x1B, 1, 7 + 30, 0 + 40);
-            fprintf(stderr, "Unknown address<BACKEND:%s> '%s:%s'\n", 
+            fprintf(stderr, "Unknown address<BACKEND:%s> '%s:%s'\n",
                     offline ? "offline" : "online",
                     uToB->peak(),
                     rtosc_argument_string(uToB->peak()));
             fprintf(stderr, "%c[%d;%d;%dm", 0x1B, 0, 7 + 30, 0 + 40);
         }
     }
+
+    if(automate.damaged) {
+        d.broadcast("/damage", "s", "/automate/");
+        automate.damaged = 0;
+    }
+
     if(events>1 && false)
         fprintf(stderr, "backend: %d events per cycle\n",events);
 
@@ -1028,7 +1372,7 @@ void Master::ShutUp()
         insefx[nefx]->cleanup();
     for(int nefx = 0; nefx < NUM_SYS_EFX; ++nefx)
         sysefx[nefx]->cleanup();
-    for(int i = 0; i < sizeof(activeNotes)/sizeof(activeNotes[0]); ++i)
+    for(int i = 0; i < int(sizeof(activeNotes)/sizeof(activeNotes[0])); ++i)
         activeNotes[i] = 0;
     vuresetpeaks();
     shutup = 0;
@@ -1074,6 +1418,8 @@ void Master::add2XML(XMLwrapper& xml)
     microtonal.add2XML(xml);
     xml.endbranch();
 
+    saveAutomation(xml, automate);
+
     for(int npart = 0; npart < NUM_MIDI_PARTS; ++npart) {
         xml.beginbranch("PART", npart);
         part[npart]->add2XML(xml);
@@ -1198,6 +1544,8 @@ void Master::getfromXML(XMLwrapper& xml)
         xml.exitbranch();
     }
 
+    loadAutomation(xml, automate);
+
     sysefx[0]->changeeffect(0);
     if(xml.enterbranch("SYSTEM_EFFECTS")) {
         for(int nefx = 0; nefx < NUM_SYS_EFX; ++nefx) {
@@ -1248,3 +1596,114 @@ void Master::getfromXML(XMLwrapper& xml)
         xml.exitbranch();
     }
 }
+
+static rtosc_version version_in_rtosc_fmt()
+{
+    return rtosc_version
+    {
+        (unsigned char) version.get_major(),
+        (unsigned char) version.get_minor(),
+        (unsigned char) version.get_revision()
+    };
+}
+
+char* Master::getXMLData()
+{
+    XMLwrapper xml;
+
+    xml.beginbranch("MASTER");
+    add2XML(xml);
+    xml.endbranch();
+
+    return xml.getXMLdata();
+}
+
+int Master::saveOSC(const char *filename)
+{
+    std::string savefile = rtosc::save_to_file(ports, this,
+                                               "ZynAddSubFX",
+                                               version_in_rtosc_fmt());
+
+    zyn::Config config;
+    zyn::SYNTH_T* synth = new zyn::SYNTH_T;
+    synth->buffersize = 256;
+    synth->samplerate = 48000;
+    synth->alias();
+
+    zyn::Master master2(*synth, &config);
+    int rval = master2.loadOSCFromStr(savefile.c_str());
+
+
+    if(rval < 0)
+    {
+        std::cerr << "invalid savefile!" << std::endl;
+        std::cerr << "complete savefile:" << std::endl;
+        std::cerr << savefile << std::endl;
+        std::cerr << "first entry that could not be parsed:" << std::endl;
+
+        for(int i = -rval + 1; savefile[i]; ++i)
+        if(savefile[i] == '\n')
+        {
+            savefile.resize(i);
+            break;
+        }
+        std::cerr << (savefile.c_str() - rval) << std::endl;
+
+        rval = -1;
+    }
+    else
+    {
+        char* xml = getXMLData(),
+            * xml2 = master2.getXMLData();
+
+        rval = strcmp(xml, xml2) ? -1 : 0;
+
+        if(rval == 0)
+        {
+            if(filename)
+            {
+                std::ofstream ofs(filename);
+                ofs << savefile;
+            }
+            else if(!filename)
+                std::cout << savefile << std::endl;
+        }
+        else
+        {
+            std::cout << savefile << std::endl;
+            std::cerr << "Can not write OSC savefile!! (see tmp1.txt and tmp2.txt)"
+                      << std::endl;
+            std::ofstream tmp1("tmp1.txt"), tmp2("tmp2.txt");
+            tmp1 << xml;
+            tmp2 << xml2;
+        }
+
+        free(xml);
+        free(xml2);
+    }
+    return rval;
+}
+
+int Master::loadOSCFromStr(const char *filename)
+{
+    return rtosc::load_from_file(filename,
+                                 ports, this,
+                                 "ZynAddSubFX", version_in_rtosc_fmt());
+}
+
+string loadfile(string fname)
+{
+    std::ifstream t(fname.c_str());
+    std::string str((std::istreambuf_iterator<char>(t)),
+                     std::istreambuf_iterator<char>());
+    return str;
+}
+
+int Master::loadOSC(const char *filename)
+{
+    int rval = loadOSCFromStr(loadfile(filename).c_str());
+    return rval < 0 ? rval : 0;
+}
+
+
+}
