@@ -20,6 +20,9 @@
 
 #define MAX_OCTAVE_SIZE 128
 #define MICROTONAL_MAX_NAME_LEN 120
+
+namespace zyn {
+
 class XMLwrapper;
 
 struct KbmInfo
@@ -154,4 +157,6 @@ class Microtonal
         const int& gzip_compression;
 };
 
+}
+
 #endif
