@@ -50,12 +50,14 @@
 #include <atomic>
 #include <list>
 
-#define errx(...)
-#define warnx(...)
+#define errx(...) {}
+#define warnx(...) {}
 #ifndef errx
 #include <err.h>
 #endif
 
+namespace zyn {
+
 using std::string;
 int Pexitprogram = 0;
 
@@ -221,55 +223,6 @@ void preparePadSynth(string path, PADnoteParameters *p, rtosc::RtData &d)
     }
 }
 
-/******************************************************************************
- *                      MIDI Serialization                                    *
- *                                                                            *
- ******************************************************************************/
-void saveMidiLearn(XMLwrapper &xml, const rtosc::MidiMappernRT &midi)
-{
-    xml.beginbranch("midi-learn");
-    for(auto value:midi.inv_map) {
-        XmlNode binding("midi-binding");
-        auto biject = std::get<3>(value.second);
-        binding["osc-path"]  = value.first;
-        binding["coarse-CC"] = to_s(std::get<1>(value.second));
-        binding["fine-CC"]   = to_s(std::get<2>(value.second));
-        binding["type"]      = "i";
-        binding["minimum"]   = to_s(biject.min);
-        binding["maximum"]   = to_s(biject.max);
-        xml.add(binding);
-    }
-    xml.endbranch();
-}
-
-void loadMidiLearn(XMLwrapper &xml, rtosc::MidiMappernRT &midi)
-{
-    using rtosc::Port;
-    if(xml.enterbranch("midi-learn")) {
-        auto nodes = xml.getBranch();
-
-        //TODO clear mapper
-
-        for(auto node:nodes) {
-            if(node.name != "midi-binding" ||
-                    !node.has("osc-path") ||
-                    !node.has("coarse-CC"))
-                continue;
-            const string path = node["osc-path"];
-            const int    CC   = atoi(node["coarse-CC"].c_str());
-            const Port  *p    = Master::ports.apropos(path.c_str());
-            if(p) {
-                printf("loading midi port...\n");
-                midi.addNewMapper(CC, *p, path);
-            } else {
-                printf("unknown midi bindable <%s>\n", path.c_str());
-            }
-        }
-        xml.exitbranch();
-    } else
-        printf("cannot find 'midi-learn' branch...\n");
-}
-
 /******************************************************************************
  *                      Non-RealTime Object Store                             *
  *                                                                            *
@@ -745,7 +698,7 @@ public:
     rtosc::UndoHistory undo;
 
     //MIDI Learn
-    rtosc::MidiMappernRT midi_mapper;
+    //rtosc::MidiMappernRT midi_mapper;
 
     //Link To the Realtime
     rtosc::ThreadLink *bToU;
@@ -797,7 +750,7 @@ class MwDataObj:public rtosc::RtData
         //Chain calls repeat the call into handle()
 
         //Forward calls send the message directly to the realtime
-        virtual void reply(const char *path, const char *args, ...)
+        virtual void reply(const char *path, const char *args, ...) override
         {
             //printf("reply building '%s'\n", path);
             va_list va;
@@ -823,7 +776,7 @@ class MwDataObj:public rtosc::RtData
                 reply(buffer);
             }
         }
-        virtual void reply(const char *msg){
+        virtual void reply(const char *msg) override{
             mwi->sendToCurrentRemote(msg);
         };
         //virtual void broadcast(const char *path, const char *args, ...){(void)path;(void)args;};
@@ -885,12 +838,12 @@ static std::vector<std::string> getFiles(const char *folder, bool finddir)
         }
 #else
         std::string darn_windows = folder + std::string("/") + std::string(fn->d_name);
-        printf("attr on <%s> => %x\n", darn_windows.c_str(), GetFileAttributes(darn_windows.c_str()));
-        printf("desired mask =  %x\n", mask);
-        printf("error = %x\n", INVALID_FILE_ATTRIBUTES);
+        //printf("attr on <%s> => %x\n", darn_windows.c_str(), GetFileAttributes(darn_windows.c_str()));
+        //printf("desired mask =  %x\n", mask);
+        //printf("error = %x\n", INVALID_FILE_ATTRIBUTES);
         bool is_dir = GetFileAttributes(darn_windows.c_str()) & FILE_ATTRIBUTE_DIRECTORY;
 #endif
-        if(finddir == is_dir)
+        if(finddir == is_dir && strcmp(".", fn->d_name))
             files.push_back(fn->d_name);
     }
 
@@ -1202,21 +1155,26 @@ static rtosc::Ports middwareSnoopPorts = {
         rEnd},
     {"save_xlz:s", 0, 0,
         rBegin;
-        const char *file = rtosc_argument(msg, 0).s;
-        XMLwrapper xml;
-        saveMidiLearn(xml, impl.midi_mapper);
-        xml.saveXMLfile(file, impl.master->gzip_compression);
+        impl.doReadOnlyOp([&]() {
+                const char *file = rtosc_argument(msg, 0).s;
+                XMLwrapper xml;
+                Master::saveAutomation(xml, impl.master->automate);
+                xml.saveXMLfile(file, impl.master->gzip_compression);
+                });
         rEnd},
     {"load_xlz:s", 0, 0,
         rBegin;
         const char *file = rtosc_argument(msg, 0).s;
         XMLwrapper xml;
         xml.loadXMLfile(file);
-        loadMidiLearn(xml, impl.midi_mapper);
+        rtosc::AutomationMgr *mgr = new rtosc::AutomationMgr(16,4,8);
+        mgr->set_ports(Master::ports);
+        Master::loadAutomation(xml, *mgr);
+        d.chain("/automate/load-blob", "b", sizeof(void*), &mgr);
         rEnd},
     {"clear_xlz:", 0, 0,
         rBegin;
-        impl.midi_mapper.clear();
+        d.chain("/automate/clear", "");
         rEnd},
     //scale file stuff
     {"load_xsz:s", 0, 0,
@@ -1389,51 +1347,51 @@ static rtosc::Ports middwareSnoopPorts = {
         impl.undo.seekHistory(+1);
         rEnd},
     //port to observe the midi mappings
-    {"midi-learn-values:", 0, 0,
-        rBegin;
-        auto &midi  = impl.midi_mapper;
-        auto  key   = keys(midi.inv_map);
-        //cc-id, path, min, max
-#define MAX_MIDI 32
-        rtosc_arg_t args[MAX_MIDI*4];
-        char        argt[MAX_MIDI*4+1] = {0};
-        int j=0;
-        for(unsigned i=0; i<key.size() && i<MAX_MIDI; ++i) {
-            auto val = midi.inv_map[key[i]];
-            if(std::get<1>(val) == -1)
-                continue;
-            argt[4*j+0]   = 'i';
-            args[4*j+0].i = std::get<1>(val);
-            argt[4*j+1]   = 's';
-            args[4*j+1].s = key[i].c_str();
-            argt[4*j+2]   = 'i';
-            args[4*j+2].i = 0;
-            argt[4*j+3]   = 'i';
-            args[4*j+3].i = 127;
-            j++;
+    //{"midi-learn-values:", 0, 0,
+    //    rBegin;
+    //    auto &midi  = impl.midi_mapper;
+    //    auto  key   = keys(midi.inv_map);
+    //    //cc-id, path, min, max
+//#define MAX_MIDI 32
+    //    rtosc_arg_t args[MAX_MIDI*4];
+    //    char        argt[MAX_MIDI*4+1] = {0};
+    //    int j=0;
+    //    for(unsigned i=0; i<key.size() && i<MAX_MIDI; ++i) {
+    //        auto val = midi.inv_map[key[i]];
+    //        if(std::get<1>(val) == -1)
+    //            continue;
+    //        argt[4*j+0]   = 'i';
+    //        args[4*j+0].i = std::get<1>(val);
+    //        argt[4*j+1]   = 's';
+    //        args[4*j+1].s = key[i].c_str();
+    //        argt[4*j+2]   = 'i';
+    //        args[4*j+2].i = 0;
+    //        argt[4*j+3]   = 'i';
+    //        args[4*j+3].i = 127;
+    //        j++;
 
-        }
-        d.replyArray(d.loc, argt, args);
-#undef  MAX_MIDI
-        rEnd},
-    {"learn:s", 0, 0,
-        rBegin;
-        string addr = rtosc_argument(msg, 0).s;
-        auto &midi  = impl.midi_mapper;
-        auto map    = midi.getMidiMappingStrings();
-        if(map.find(addr) != map.end())
-            midi.map(addr.c_str(), false);
-        else
-            midi.map(addr.c_str(), true);
-        rEnd},
-    {"unlearn:s", 0, 0,
-        rBegin;
-        string addr = rtosc_argument(msg, 0).s;
-        auto &midi  = impl.midi_mapper;
-        auto map    = midi.getMidiMappingStrings();
-        midi.unMap(addr.c_str(), false);
-        midi.unMap(addr.c_str(), true);
-        rEnd},
+    //    }
+    //    d.replyArray(d.loc, argt, args);
+//#undef  MAX_MIDI
+    //    rEnd},
+    //{"learn:s", 0, 0,
+    //    rBegin;
+    //    string addr = rtosc_argument(msg, 0).s;
+    //    auto &midi  = impl.midi_mapper;
+    //    auto map    = midi.getMidiMappingStrings();
+    //    if(map.find(addr) != map.end())
+    //        midi.map(addr.c_str(), false);
+    //    else
+    //        midi.map(addr.c_str(), true);
+    //    rEnd},
+    //{"unlearn:s", 0, 0,
+    //    rBegin;
+    //    string addr = rtosc_argument(msg, 0).s;
+    //    auto &midi  = impl.midi_mapper;
+    //    auto map    = midi.getMidiMappingStrings();
+    //    midi.unMap(addr.c_str(), false);
+    //    midi.unMap(addr.c_str(), true);
+    //    rEnd},
     //drop this message into the abyss
     {"ui/title:", 0, 0, [](const char *msg, RtData &d) {}},
     {"quit:", 0, 0, [](const char *, RtData&) {Pexitprogram = 1;}},
@@ -1480,10 +1438,6 @@ static rtosc::Ports middlewareReplyPorts = {
         if(impl.recording_undo)
             impl.undo.recordEvent(msg);
         rEnd},
-    {"midi-use-CC:i", 0, 0,
-        rBegin;
-        impl.midi_mapper.useFreeID(rtosc_argument(msg, 0).i);
-        rEnd},
     {"broadcast:", 0, 0, rBegin; impl.broadcast = true; rEnd},
     {"forward:", 0, 0, rBegin; impl.forward = true; rEnd},
 };
@@ -1508,8 +1462,8 @@ MiddleWareImpl::MiddleWareImpl(MiddleWare *mw, SYNTH_T synth_,
 {
     bToU = new rtosc::ThreadLink(4096*2*16,1024/16);
     uToB = new rtosc::ThreadLink(4096*2*16,1024/16);
-    midi_mapper.base_ports = &Master::ports;
-    midi_mapper.rt_cb      = [this](const char *msg){handleMsg(msg);};
+    //midi_mapper.base_ports = &Master::ports;
+    //midi_mapper.rt_cb      = [this](const char *msg){handleMsg(msg);};
     if(preferrred_port != -1)
         server = lo_server_new_with_proto(to_s(preferrred_port).c_str(),
                                           LO_UDP, liblo_error_cb);
@@ -2165,3 +2119,5 @@ PresetsStore& MiddleWare::getPresetsStore()
 {
     return impl->presetsstore;
 }
+
+}
