@@ -29,13 +29,15 @@
 #include <sched.h>
 #endif
 
-#define errx(...)
+#define errx(...) {}
 #ifndef errx
 #include <err.h>
 #endif
 
 #include <rtosc/rtosc.h>
 
+namespace zyn {
+
 bool isPlugin = false;
 
 prng_t prng_state = 0x1234;
@@ -225,3 +227,5 @@ char *rtosc_splat(const char *path, std::set<std::string> v)
     rtosc_amessage(buf, len, path, argT, arg);
     return buf;
 }
+
+}
