@@ -23,16 +23,17 @@
 #include <rtosc/ports.h>
 #include <rtosc/port-sugar.h>
 
+namespace zyn {
+
+extern bool isPlugin;
+bool fileexists(const char *filename);
+
 using std::min;
 using std::max;
 
 //Velocity Sensing function
 extern float VelF(float velocity, unsigned char scaling);
 
-extern bool isPlugin;
-
-bool fileexists(const char *filename);
-
 #define N_DETUNE_TYPES 4 //the number of detune types
 extern float getdetune(unsigned char type,
                        unsigned short int coarsedetune,
@@ -152,11 +153,6 @@ char *rtosc_splat(const char *path, std::set<std::string>);
 #define rParamZyn(name, ...) \
   {STRINGIFY(name) "::i",  rProp(parameter) rMap(min, 0) rMap(max, 127) DOC(__VA_ARGS__), NULL, rParamICb(name)}
 
-#define rSelf(type) \
-{"self:", rProp(internal) rMap(class, type) rDoc("port metadata"), 0, \
-    [](const char *, rtosc::RtData &d){ \
-        d.reply(d.loc, "b", sizeof(d.obj), &d.obj);}}\
-
 #define rPresetType \
 {"preset-type:", rProp(internal) rDoc("clipboard type of object"), 0, \
     [](const char *, rtosc::RtData &d){ \
@@ -181,4 +177,8 @@ rPresetType, \
         rObject &o = *(rObject*)d.obj;\
         o.pasteArray(paste,field);}}
 
+}
+
+#define rUnit(x) rMap(unit, x)
+
 #endif
