@@ -23,6 +23,7 @@
 #include <rtosc/ports.h>
 using namespace rtosc;
 
+namespace zyn {
 
 #define rObject LFOParams
 #undef rChangeCb
@@ -41,13 +42,15 @@ static const rtosc::Ports _ports = {
                 exp1, exp2), "Shape of LFO"),
     rParamZyn(Prandomness, rShort("a.r."), rSpecial(disable),
             "Amplitude Randomness (calculated uniformly at each cycle)"),
-    rParamZyn(Pfreqrand, rShort("f.r."), rSpecial(disable),
+    rParamZyn(Pfreqrand, rShort("f.r."), rSpecial(disable), rDefault(0),
             "Frequency Randomness (calculated uniformly at each cycle)"),
     rParamZyn(Pdelay, rShort("delay"), rSpecial(disable), "Delay before LFO start\n"
             "0..4 second delay"),
     rToggle(Pcontinous, rShort("c"), "Enable for global operation"),
-    rParamZyn(Pstretch, rShort("str"), rCentered, "Note frequency stretch"),
-
+    rParamZyn(Pstretch, rShort("str"), rCentered, rDefault(64),
+        "Note frequency stretch"),
+// these are currently not yet implemented at must be hidden therefore
+#ifdef DEAD_PORTS
     //Float valued aliases
     {"delay::f", rProp(parameter) rMap(units, ms) rLog(0,4000), 0,
         rBegin;
@@ -57,6 +60,7 @@ static const rtosc::Ports _ports = {
     {"period::f", rProp(parameter) rMap(units, ms) rPseudoLog(0.10, 1500.0), 0,
         rBegin;
         rEnd},
+#endif
 };
 #undef rPseudoLog
 #undef rBegin
@@ -173,3 +177,5 @@ void LFOParams::paste(LFOParams &x)
     }
 }
 #undef COPY
+
+}
