@@ -15,6 +15,9 @@
 #define PRESETS_H
 
 #include "../globals.h"
+
+namespace zyn {
+
 class PresetsStore;
 
 /**Presets and Clipboard management*/
@@ -40,4 +43,6 @@ class Presets
         //virtual void defaults() = 0;
 };
 
+}
+
 #endif
