@@ -52,7 +52,7 @@ public:
                   thread.start(middleware);
           }
 
-          void updateMiddleWare(MiddleWare* const mw) noexcept
+	  void updateMiddleWare(zyn::MiddleWare* const mw) noexcept
           {
               middleware = mw;
           }
@@ -60,7 +60,7 @@ public:
       private:
           const bool wasRunning;
           MiddleWareThread& thread;
-          MiddleWare* middleware;
+	  zyn::MiddleWare* middleware;
 
           DISTRHO_PREVENT_HEAP_ALLOCATION
           DISTRHO_DECLARE_NON_COPY_CLASS(ScopedStopper)
@@ -70,7 +70,7 @@ public:
           : Thread("ZynMiddleWare"),
             middleware(nullptr) {}
 
-      void start(MiddleWare* const mw) noexcept
+      void start(zyn::MiddleWare* const mw) noexcept
       {
           middleware = mw;
           startThread();
@@ -96,7 +96,7 @@ protected:
     }
 
 private:
-    MiddleWare* middleware;
+    zyn::MiddleWare* middleware;
 
     DISTRHO_DECLARE_NON_COPY_CLASS(MiddleWareThread)
 };
@@ -115,8 +115,6 @@ public:
           oscPort(0),
           middlewareThread(new MiddleWareThread())
     {
-        config.init();
-
         synth.buffersize = static_cast<int>(getBufferSize());
         synth.samplerate = static_cast<uint>(getSampleRate());
 
@@ -191,7 +189,9 @@ protected:
     */
     uint32_t getVersion() const noexcept override
     {
-    return d_version(version.get_major(), version.get_minor(), version.get_revision());
+        return d_version(zyn::version.get_major(),
+                         zyn::version.get_minor(),
+                         zyn::version.get_revision());
     }
 
    /**
@@ -224,6 +224,15 @@ protected:
             parameter.ranges.def = 0.0f;
             break;
         }
+        if(kParamSlot1 <= index && index <= kParamSlot16) {
+            parameter.hints  = kParameterIsAutomable;
+            parameter.name   = ("Slot " + zyn::to_s(index-kParamSlot1 + 1)).c_str();
+            parameter.symbol = ("slot"  + zyn::to_s(index-kParamSlot1 + 1)).c_str();
+            parameter.unit   = "";
+            parameter.ranges.min = 0.0f;
+            parameter.ranges.max = 1.0f;
+            parameter.ranges.def = 0.5f;
+        }
     }
 
    /**
@@ -236,9 +245,11 @@ protected:
         {
         case kParamOscPort:
             return oscPort;
-        default:
-            return 0.0f;
         }
+        if(kParamSlot1 <= index && index <= kParamSlot16) {
+            return master->automate.getSlot(index - kParamSlot1);
+        }
+        return 0.0f;
     }
 
    /**
@@ -247,9 +258,12 @@ protected:
       When a parameter is marked as automable, you must ensure no non-realtime operations are performed.
       @note This function will only be called for parameter inputs.
     */
-    void setParameterValue(uint32_t /*index*/, float /*value*/) noexcept override
+    void setParameterValue(uint32_t index, float value) noexcept override
     {
         // only an output port for now
+        if(kParamSlot1 <= index && index <= kParamSlot16) {
+            master->automate.setSlot(index - kParamSlot1, value);
+        }
     }
 
    /* --------------------------------------------------------------------------------------------------------
@@ -303,6 +317,11 @@ protected:
         const MiddleWareThread::ScopedStopper mwss(*middlewareThread);
         const MutexLocker cml(mutex);
 
+        if(key && strlen(key) > 1000 && (!value || strlen(value) < 1000)) {
+            //Loading a Jackoo VST File
+            value = key;
+        }
+
         master->defaults();
         master->putalldata(value);
         master->applyparameters();
@@ -388,12 +407,23 @@ protected:
                 master->setController(channel, control, value);
             } break;
 
+            case 0xC0: {
+                const int program = midiEvent.data[1];
+
+                for(int i=0; i < NUM_MIDI_PARTS; ++i) {
+                    //set the program of the parts assigned to the midi channel
+                    if(master->part[i]->Prcvchn == channel) {
+                        middleware->pendingSetProgram(i, program);
+                    }
+                }
+            } break;
+
             case 0xE0: {
                 const uint8_t lsb = midiEvent.data[1];
                 const uint8_t msb = midiEvent.data[2];
                 const int   value = ((msb << 7) | lsb) - 8192;
 
-                master->setController(channel, C_pitchwheel, value);
+                master->setController(channel, zyn::C_pitchwheel, value);
             } break;
             }
         }
@@ -459,10 +489,10 @@ protected:
     }
 
 private:
-    Config      config;
-    Master*     master;
-    MiddleWare* middleware;
-    SYNTH_T     synth;
+    zyn::Config      config;
+    zyn::Master*     master;
+    zyn::MiddleWare* middleware;
+    zyn::SYNTH_T     synth;
 
     Mutex mutex;
     char* defaultState;
@@ -481,7 +511,7 @@ private:
 
     void _initMaster()
     {
-        middleware = new MiddleWare(std::move(synth), &config);
+	middleware = new zyn::MiddleWare(std::move(synth), &config);
         middleware->setUiCallback(__uiCallback, this);
         middleware->setIdleCallback(__idleCallback, this);
         _masterChangedCallback(middleware->spawnMaster());
@@ -507,13 +537,13 @@ private:
    /* --------------------------------------------------------------------------------------------------------
     * ZynAddSubFX Callbacks */
 
-    void _masterChangedCallback(Master* m)
+    void _masterChangedCallback(zyn::Master* m)
     {
         master = m;
         master->setMasterChangedCallback(__masterChangedCallback, this);
     }
 
-    static void __masterChangedCallback(void* ptr, Master* m)
+    static void __masterChangedCallback(void* ptr, zyn::Master* m)
     {
         ((ZynAddSubFX*)ptr)->_masterChangedCallback(m);
     }
@@ -554,7 +584,7 @@ START_NAMESPACE_DISTRHO
 
 Plugin* createPlugin()
 {
-    ::isPlugin = true;
+    zyn::isPlugin = true;
     return new ZynAddSubFX();
 }
 
@@ -563,9 +593,10 @@ END_NAMESPACE_DISTRHO
 /* ------------------------------------------------------------------------------------------------------------
  * Dummy variables and functions for linking purposes */
 
+namespace zyn {
 class WavFile;
 namespace Nio {
-   void masterSwap(Master*){}
+   void masterSwap(zyn::Master*){}
    bool setSource(std::string){return true;}
    bool setSink(std::string){return true;}
    std::set<std::string> getSources(void){return std::set<std::string>();}
@@ -576,3 +607,4 @@ namespace Nio {
    void waveStart(){}
    void waveStop(){}
 }
+}
