@@ -27,6 +27,8 @@
 
 #define OSCIL_SMP_EXTRA_SAMPLES 5
 
+namespace zyn {
+
 /**The "additive" synthesizer*/
 class ADnote:public SynthNote
 {
@@ -51,6 +53,11 @@ class ADnote:public SynthNote
         virtual SynthNote *cloneLegato(void) override;
     private:
 
+        void setupVoice(int nvoice);
+        int  setupVoiceUnison(int nvoice);
+        void setupVoiceDetune(int nvoice);
+        void setupVoiceMod(int nvoice);
+
         /**Changes the frequency of an oscillator.
          * @param nvoice voice to run computations on
          * @param in_freq new frequency*/
@@ -317,4 +324,6 @@ class ADnote:public SynthNote
         float bandwidthDetuneMultiplier;
 };
 
+}
+
 #endif
