@@ -27,11 +27,12 @@
 #include "../Misc/Util.h"
 #include "../globals.h"
 #include "../UI/NSM.H"
+using namespace std;
+using namespace zyn;
+
 class NSM_Client *nsm = 0;
 MiddleWare *middleware = 0;
 
-using namespace std;
-
 char *instance_name=(char*)"";
 
 #define NUM_MIDDLEWARE 3
@@ -183,14 +184,48 @@ class MessageTest:public CxxTest::TestSuite
             mw->transmitMsg("/learn", "s", "/Pvolume");
             mw->transmitMsg("/virtual_midi_cc", "iii", 0, 23, 108);
 
+            //param is at default until rt-thread is run
+            TS_ASSERT_EQUALS(ms->Pvolume, 80);
+
+
             //Perform a learning operation
+            run_realtime();
 
-            run_realtime(); //1. runs learning and identifies a CC to bind
-            mw->tick();     //2. produces new binding table
-            run_realtime(); //3. applies new binding table
+            //Verify binding affects control
+            TS_ASSERT_EQUALS(ms->Pvolume, 108);
 
+
+            printf("# Trying to save automations\n");
+            start_realtime();
             mw->transmitMsg("/save_xlz", "s", "test-midi-learn.xlz");
+            stop_realtime();
+
+            //Verify that some file exists
+            printf("# Verifying file exists\n");
+            FILE *f = fopen("test-midi-learn.xlz", "r");
+            TS_ASSERT(f);
+
+            if(f)
+                fclose(f);
+
+            printf("# Clearing automation\n");
+            //Clear out state
+            mw->transmitMsg("/clear_xlz", "");
+            //Send dummy message
+            mw->transmitMsg("/virtual_midi_cc", "iii", 0, 23, 27);
+            run_realtime();
+
+            //Verify automation table is clear
+            TS_ASSERT_EQUALS(ms->Pvolume, 108);
+
+            printf("# Loading automation\n");
             mw->transmitMsg("/load_xlz", "s", "test-midi-learn.xlz");
+            //Send message
+            mw->transmitMsg("/virtual_midi_cc", "iii", 0, 23, 28);
+            run_realtime();
+
+            //Verify automation table is restored
+            TS_ASSERT_EQUALS(ms->Pvolume, 28);
         }
 
         void testLfoPaste(void)
@@ -242,6 +277,53 @@ class MessageTest:public CxxTest::TestSuite
             TS_ASSERT_EQUALS(field2, 35);
         }
 
+        void testFilterDepricated(void)
+        {
+            vector<string> v = {"Pfreq", "Pfreqtrack", "Pgain", "Pq"};
+            for(int i=0; i<v.size(); ++i) {
+                string path = "/part0/kit0/adpars/GlobalPar/GlobalFilter/"+v[i];
+                for(int j=0; j<128; ++j) {
+                    mw->transmitMsg(path.c_str(), "i", j); //Set
+                    mw->transmitMsg(path.c_str(), ""); //Get
+                }
+
+            }
+            while(ms->uToB->hasNext()) {
+                const char *msg = ms->uToB->read();
+                //printf("RT: handling <%s>\n", msg);
+                ms->applyOscEvent(msg);
+            }
+
+            int id = 0;
+            int state = 0;
+            int value = 0;
+            // 0 - broadcast
+            // 1 - true value     (set)
+            // 2 - expected value (get)
+            while(ms->bToU->hasNext()) {
+                const char *msg = ms->bToU->read();
+                if(state == 0) {
+                    TS_ASSERT_EQUALS(rtosc_narguments(msg), 0);
+                    state = 1;
+                } else if(state == 1) {
+                    TS_ASSERT_EQUALS(rtosc_narguments(msg), 1);
+                    value = rtosc_argument(msg, 0).i;
+                    state = 2;
+                } else if(state == 2) {
+                    int val = rtosc_argument(msg, 0).i;
+                    if(value != val) {
+                        printf("%s - %d should equal %d\n", msg, value, val);
+                        TS_ASSERT(0);
+                    }
+                    state = 0;
+                }
+
+                //printf("Message #%d %s:%s\n", id++, msg, rtosc_argument_string(msg));
+                //if(rtosc_narguments(msg))
+                //    printf("        %d\n", rtosc_argument(msg, 0).i);
+            }
+        }
+
 
     private:
         SYNTH_T     *synth;
